function hasProperty(context, field) {
    if (typeof context === 'number' || typeof context === 'string') {
        return context[field] || Object.prototype.hasOwnProperty.call(context, field);
    }
    if (!context) {
        return false;
    }
    return field in context;
}
const matchProperty = (value, notInContext) => ({ property, context }) => value === property && (notInContext ? !hasProperty(context, property) : true);
const matchStartWith = (value) => ({ property, context }) => property.indexOf(value) === 0 && !(property in context) && property.length > value.length;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function getter(base, property) {
    // get(1)
    if (typeof property === 'number') {
        return base[property];
    }
    const letter = property.charCodeAt(0);
    const isUpper = letter < 91;
    const ret = base[property];
    if (ret !== undefined) {
        return ret;
    }
    if (isUpper) {
        // Address => address
        property = String.fromCharCode(letter).toLowerCase() + property.slice(1);
    }
    if (!isUpper) {
        // address => Address
        property = String.fromCharCode(letter).toUpperCase() + property.slice(1);
    }
    return base[property];
}
function getSize(obj) {
    if (Array.isArray(obj)) {
        return obj.length;
    }
    if (typeof obj === 'object' && obj !== null) {
        return Object.keys(obj).length;
    }
    return undefined;
}
const handlers = {
    // $foo.get('bar')
    get: {
        match: matchProperty('get', true),
        resolve: ({ context, params }) => getter(context, params[0]),
    },
    // $foo.set('a', 'b')
    set: {
        match: matchProperty('set', true),
        resolve: ({ context, params }) => {
            context[params[0]] = params[1];
            return '';
        },
    },
    // getAddress()
    getValue: {
        match: matchStartWith('get'),
        resolve: ({ context, property }) => getter(context, property.slice(3)),
    },
    isValue: {
        match: matchStartWith('is'),
        resolve: ({ context, property }) => getter(context, property.slice(2)),
    },
    // $page.setName(123)
    setValue: {
        match: matchStartWith('set'),
        resolve: ({ context, property, params }) => {
            context[property.slice(3)] = params[0];
            // set value will not output anything
            context.toString = function () {
                return '';
            };
            return context;
        },
    },
    keySet: {
        match: matchProperty('keySet', true),
        resolve: ({ context }) => Object.keys(context),
    },
    entrySet: {
        match: matchProperty('entrySet', true),
        resolve: ({ context }) => Object.keys(context).map((key) => ({
            key,
            value: context[key],
        })),
    },
    size: {
        match: matchProperty('size', true),
        resolve: ({ context }) => getSize(context),
    },
    put: {
        match: matchProperty('put', true),
        resolve: ({ context, params }) => (context[params[0]] = params[1]),
    },
    add: {
        match: matchProperty('add', true),
        resolve: ({ context, params }) => {
            if (typeof context.push !== 'function') {
                return;
            }
            return context.push(params[0]);
        },
    },
    remove: {
        match: matchProperty('remove', true),
        resolve: ({ context, params }) => {
            if (Array.isArray(context)) {
                let index = params[0];
                if (typeof index !== 'number') {
                    index = context.indexOf(params[0]);
                }
                const ret = context[index];
                context.splice(index, 1);
                return ret;
            }
            if (typeof context === 'object' && context !== null) {
                const ret = context[params[0]];
                delete context[params[0]];
                return ret;
            }
            return undefined;
        },
    },
    subList: {
        match: matchProperty('subList', true),
        resolve: ({ context, params }) => context.slice(params[0], params[1]),
    },
};
const keys = Object.keys(handlers);
export default keys.map((key) => ({
    uid: 'system: ' + key,
    match: handlers[key].match,
    resolve: handlers[key].resolve,
}));
//# sourceMappingURL=methods.js.map